define("CallsMessagePublisherPage", ["ConfigurationConstants", "LookupQuickAddMixin", "ESNHtmlEditModule",
		"css!CallsMessagePublisherModule"],
	function(ConfigurationConstants) {
		return {
			entitySchemaName: "Activity",
			mixins: {
				LookupQuickAddMixin: "Terrasoft.LookupQuickAddMixin"
			},
			attributes: {
				"EntitiesList": {
					"dataValueType": Terrasoft.DataValueType.COLLECTION,
					"type": Terrasoft.ViewModelColumnType.VIRTUAL_COLUMN
				},
				//
				"CallDirectionsList": {
					"dataValueType": Terrasoft.DataValueType.COLLECTION
				},
				"Contact": {
					"lookupListConfig": {
						"columns": ["Account"]
					}
				}
			},
			methods: {
				init: function() {
					this.callParent(arguments);
					this.setDefaultValues();
				},
				onLookupDataLoaded: function(config) {
					this.callParent(arguments);
					config.isLookupEdit = true;
					this.mixins.LookupQuickAddMixin.onLookupDataLoaded.call(this, config);
				},
				getServiceConfig: function() {
					return {
						className: "Terrasoft.Configuration.CallsMessagePublisher"
					};
				},
				onPublished: function() {
					this.callParent(arguments);
					this.set("Body", null);
				},
				getPublishMaskCaption: function() {
					return this.get("Resources.Strings.PublishMaskCaption");
				},
				prepareCallDirectionsList: function(filter, list) {
					if (!list) {
						return;
					}
					list.clear();
					this.getCallDirections(list);
				},
				getCallDirectionQuery: function() {
					var query = this.Ext.create("Terrasoft.EntitySchemaQuery", {
						rootSchemaName: "CallDirection"
					});
					query.addColumn("Name");
					var callDirectionFilter = this.getCallDirectionFilter(query);
					query.filters.add("callDirectionFilter", callDirectionFilter);
					return query;
				},
				getCallDirectionFilter: function(query) {
					if (!query) {
						return null;
					}
					var notDefinedDirection = -1;
					return query.createColumnFilterWithParameter(Terrasoft.ComparisonType.NOT_EQUAL,
						"Code", notDefinedDirection);
				},
				getCallDirections: function(list) {
					var query = this.getCallDirectionQuery();
					query.getEntityCollection(function(result) {
						this.getCallDirectionsHandler(result, list);
					}, this);
				},
				getCallDirectionsHandler: function(result, list) {
					if (result.success) {
						var self = this;
						var collection = {};
						var items = result.collection.getItems();
						items.forEach(function(item) {
							self.addCallDirectionToCollection(item, collection);
						});
						list.loadAll(collection);
					}
				},
				addCallDirectionToCollection: function(item, collection) {
					var id = item.get("Id");
					var name = item.get("Name");
					collection[id] = {
						value: id,
						displayValue: name
					};
				},
				setDefaultValues: function() {
					this.set("CallDirectionsList", this.Ext.create("Terrasoft.Collection"));
					this.setDefaultContact();
					this.setDefaultCallDirection();
				},
				onDashboardRealoaded: function() {
					this.setDefaultValues();
				},
				getContactLookupValue: function(config) {
					var result = this.Ext.isObject(config) ? {
						Account: config.Account,
						displayValue: config.displayValue,
						value: config.value
					} : null;
					return result;
				},
				setDefaultContact: function() {
					var config = this.getListenerRecordData();
					if (!config) {
						return;
					}
					var additionalInfo = config.additionalInfo;
					var contact = this.getContactLookupValue(additionalInfo && additionalInfo.contact);
					this.set("Contact", contact);
				},
				setDefaultCallDirection: function() {
					var query = this.Ext.create("Terrasoft.EntitySchemaQuery", {
						rootSchemaName: "CallDirection"
					});
					query.addColumn("Name");
					var recordId = ConfigurationConstants.Activity.ActivityCallDirection.Incoming;
					query.getEntity(recordId, function(result) {
						this.setDefaultCallDirectionQueryHandler(result);
					}, this);
				},
				setDefaultCallDirectionQueryHandler: function(result) {
					if (result.success) {
						var id = result.entity.get("Id");
						var name = result.entity.get("Name");
						var defaultCallDirection = {
							value: id,
							displayValue: name
						};
						this.set("CallDirection", defaultCallDirection);
					}
				},
				getPublishDataItem: function(itemName, itemValue) {
					if (!itemName || !itemValue) {
						return;
					}
					return {"Key": itemName, "Value": itemValue};
				},
				getRelationSchemaForPublishData: function() {
					var config = this.getListenerRecordData();
					if (!config || !config.relationSchemaName || !config.relationSchemaRecordId) {
						return null;
					}
					var relationSchemaName = config.relationSchemaName + "Id";
					var relationSchemaRecordId = config.relationSchemaRecordId;
					return this.getPublishDataItem(relationSchemaName, relationSchemaRecordId);
				},
				getContactForPublishData: function() {
					var contact = this.get("Contact");
					if (!contact || !contact.value) {
						return null;
					}
					var contactValue = contact.value;
					return this.getPublishDataItem("ContactId", contactValue);
				},
				getAccountForPublishData: function() {
					var contact = this.get("Contact");
					if (!contact || !contact.Account || !contact.Account.value) {
						return null;
					}
					return this.getPublishDataItem("AccountId", contact.Account.value);
				},
				getCallDirectionToPublishData: function() {
					var callDirection = this.get("CallDirection");
					if (!callDirection || !callDirection.value) {
						return null;
					}
					return this.getPublishDataItem("CallDirectionId", callDirection.value);
				},
				onNewCardSaved: function() {
					if (!this.get("Body")) {
						return;
					}
					this.publishMessage();
				},
				getPublishData: function() {
					var publishData = [];
					var contact = this.getContactForPublishData();
					if (contact) {
						publishData.push(contact);
					}
					var account = this.getAccountForPublishData();
					if (account) {
						publishData.push(account);
					}
					var relationSchema = this.getRelationSchemaForPublishData();
					if (relationSchema) {
						var itemIndex = publishData.findIndex(function(item) {
							return item.Key === relationSchema.Key;
						}, this);
						if (itemIndex === -1) {
							publishData.push(relationSchema);
						}
					}
					var callDirection = this.getCallDirectionToPublishData();
					if (callDirection) {
						publishData.push(callDirection);
					}
					var callCategory = ConfigurationConstants.Activity.ActivityCategory.Call;
					publishData.push(this.getPublishDataItem("ActivityCategoryId", callCategory));
					var body = this.getClearBody(this.get("Body"));
					publishData.push(this.getPublishDataItem("Body", body));
					publishData.push(this.getPublishDataItem("DetailedResult", body));
					var title = this.getCombinedTitle();
					publishData.push(this.getPublishDataItem("Title", title));
					publishData.push(this.getPublishDataItem("ResultId",
						ConfigurationConstants.Activity.ActivityResult.Completed));
					publishData.push(this.getPublishDataItem("StatusId",
						ConfigurationConstants.Activity.Status.Done));
					return publishData;
				},
				getCombinedTitle: function() {
					var contact = this.get("Contact");
					var callDirection = this.get("CallDirection");
					var directionMessage;
					var undefinedCallDirectionMessage;
					if (callDirection.value === ConfigurationConstants.Activity.ActivityCallDirection.Incoming) {
						directionMessage = this.get("Resources.Strings.IncomingCallString");
						undefinedCallDirectionMessage = this.get("Resources.Strings.UndefinedIncomingCallString");
					} else if (callDirection.value ===
						ConfigurationConstants.Activity.ActivityCallDirection.Outgoing) {
						directionMessage = this.get("Resources.Strings.OutgoingCallString");
						undefinedCallDirectionMessage = this.get("Resources.Strings.UndefinedOutgoingCallString");
					}
					if (!contact) {
						return undefinedCallDirectionMessage;
					}
					return  this.Ext.String.format("{0} {1}", directionMessage, contact.displayValue);
				},
				getClearBody: function(body) {
					if (typeof(body) === "string") {
						return body.replace(/<\/?[^>]+(>|$)/g, "").replace(/\n\n/g, "\n");
					}
				}
			},
			diff: /**SCHEMA_DIFF*/[
				{
					"operation": "insert",
					"propertyName": "items",
					"parentName": "LayoutContainer",
					"name": "MainGridLayout",
					"values": {
						"itemType": this.Terrasoft.ViewItemType.GRID_LAYOUT,
						"items": []
					},
					"index": 0
				},
				{
					"operation": "insert",
					"name": "Contact",
					"parentName": "MainGridLayout",
					"propertyName": "items",
					"values": {
						"layout": {
							"column": 0,
							"row": 0,
							"colSpan": 11
						},
						"labelConfig": {
							"classes": ["calls-message-contact"]
						},
						"bindTo": "Contact"
					}
				},
				{
					"operation": "insert",
					"name": "CallDirection",
					"parentName": "MainGridLayout",
					"propertyName": "items",
					"values": {
						"layout": {
							"column": 15,
							"row": 0,
							"colSpan": 9
						},
						"labelConfig": {
							"classes": ["call-direction-message-contact"]
						},
						"bindTo": "CallDirection",
						"contentType": this.Terrasoft.ContentType.ENUM,
						"controlConfig": {
							"prepareList": {
								"bindTo": "prepareCallDirectionsList"
							}
						}
					}
				},
				{
					"operation": "insert",
					"name": "CallsMessagePublisherEdit",
					"parentName": "MainGridLayout",
					"propertyName": "items",
					"values": {
						"layout": {
							"column": 0,
							"row": 1,
							"colSpan": 24,
							"rowSpan": 2
						},
						"className": "Terrasoft.ESNHtmlEdit",
						"itemType": this.Terrasoft.ViewItemType.MODEL_ITEM,
						"dataValueType": this.Terrasoft.DataValueType.TEXT,
						"contentType": this.Terrasoft.ContentType.RICH_TEXT,
						"labelConfig": {
							"visible": false
						},
						"value": {
							"bindTo": "Body"
						},
						"placeholder": {
							"bindTo": "Resources.Strings.WriteCommentHint"
						},
						"markerValue": "CallsMessagePublisherEdit",
						"height": "100px",
						"autoGrow": true,
						"autoGrowMinHeight": 100
					}
				}
			]/**SCHEMA_DIFF*/
		};
	}
);
